/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentFactory;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.mapping.DefaultMappingContext;
import org.objectweb.kilim.model.mapping.JavaLoggerMapper;
import org.objectweb.kilim.model.mapping.JavaRuntimeMapper;
import org.objectweb.kilim.model.mapping.MappingContext;
import org.objectweb.kilim.model.mapping.TextAreaMapper;
import org.objectweb.kilim.model.mapping.TreeModelMapper;
import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;
import org.objectweb.kilim.model.instanciation.NullInstanciationMger;
import org.objectweb.kilim.tools.KilimComponentViewer;
import org.objectweb.kilim.tools.KilimTemplateViewer;
import org.objectweb.kilim.tools.KilimTraceTreeModel;
import org.objectweb.kilim.tools.KilimTraceTreeViewer;

import org.xml.sax.SAXException;

/**
 * @author delpiano
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class QuotesWireTests extends TestCase {

	private ResourceRepository rep;
	private ResourceRepository rep1;

	/**
	 * Constructor for QuotesWireTests.
	 * @param arg0 for junit
	 * @throws SAXException should not happen
	 * @throws ParserConfigurationException should not happen
	 */
	public QuotesWireTests(String arg0) throws SAXException, ParserConfigurationException {
		super(arg0);
		TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
		rep = new ResourceRepository(parser);
		rep.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
		rep1 = new ResourceRepository1(parser);
		rep1.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
	}

	/**
	 * Method testQuotes1.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testQuotes1() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("quoteswire/TradingConsoleV3");
			//KilimTemplateViewer.viewTemplate(ts0)
			//this specific strategy should be defined in order to obtain the expected behaviour ...
			InstanciationStrategy strgy = new DefaultInstanciationStrategy();			
			TextAreaMapper textMapper = new TextAreaMapper(new JavaRuntimeMapper());
			TreeModelMapper treeMapper = new TreeModelMapper(textMapper);
			KilimTraceTreeModel treeModel = treeMapper.getTraceTree();
			//Component compo = ComponentFactory.newComponent(ts0, treeMapper, new DefaultMappingContext(), strgy);
			Component compo = ComponentFactory.newComponent(ts0);
			ComponentInterface interf = compo.getInterface("publishers");
			Object res = interf.getValue();
			interf = compo.getInterface("trader's workspace");
			res = interf.getValue();
			Thread.sleep(5000);
			//KilimComponentViewer.viewComponent(compo);
			//KilimTraceTreeViewer.viewTrace(treeModel, textMapper.getTextArea());		

		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}
	}
	
	/**
	 * Method plugsAndSlots.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	private void plugsAndSlots(String tmpl_nm) throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			//this specific strategy should be defined in order to obtain the expected behaviour ...
			InstanciationStrategy strgy = new DefaultInstanciationStrategy();
			Component compo = org.objectweb.kilim.helpers.KilimHelper.newComponent(tmpl_nm , this.getClass(), strgy);			
			ComponentInterface interf = compo.getInterface("console");
			Object res = interf.getValue();
			Thread.sleep(5000);	
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
	
	/**
	 * Method testQuotes2.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testQuotes2() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		plugsAndSlots("quoteswire/ExtensibleConsole");
	}
	
	/**
	 * Method testQuotes3.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testQuotes3() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		plugsAndSlots("quoteswire/ExtensibleConsole2");
	}
	
	/**
	 * Method testQuotes4.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testQuotes4() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		plugsAndSlots("quoteswire/AnotherExtensibleConsole");
	}
	/**
	 * Method testQuotes5.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testQuotes5() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		plugsAndSlots("quoteswire/AnotherExtensibleConsole2");
	}	
	
	/**
	 * Method testQuotes6.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 */
	public void testQuotes6() throws ResourceNotFoundException, SAXException, ParserConfigurationException {	
		try {
			TemplateDescription ts0 = rep1.getTemplateDescription("quoteswire/channels/SimpleChannelWithSlot");		
			//KilimTemplateViewer.viewTemplate(ts0);
			Component compo = ComponentFactory.newComponent(ts0);			
			ComponentInterface interf = compo.getInterface("channel");
			Object res = interf.getValue();
			Thread.sleep(5000);
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}

}
