/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import java.util.Iterator;
import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.KilimConfiguration;
import org.objectweb.kilim.description.Property;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentFactory;
import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;
import org.objectweb.kilim.model.instanciation.NullInstanciationMger;

import org.xml.sax.SAXException;

/**
 * @author horn, delpiano
 * Kilim2 xml parser tests
 */
public class TemplateParserTest extends TestCase {
	
	private ResourceRepository rep;
	private ResourceRepository rep1;
	
	/**
	 * Constructor for TemplateParserTest.
	 * @param arg0 for junit
	 * @throws SAXException should not happen
	 * @throws ParserConfigurationException should not happen
	 */
	public TemplateParserTest(String arg0) throws SAXException, ParserConfigurationException {
		super(arg0);
		TemplateDescriptionParser parser = new TemplateDescriptionParser(true);
		rep = new ResourceRepository(parser);
		rep.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
		rep1 = new ResourceRepository1(parser);
		rep1.setResourceLoader(new ClassLoaderResourceLoader(this.getClass().getClassLoader()));
	}
	
		/**
	 * Basic parsing tests
	 * @throws ResourceNotFoundException should not happen
	 * @throws ParserConfigurationException should not happen
	 * @throws SAXException should not happen
	 */
	public void testPort1() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("test-port1");
			Iterator iter = ts0.getProperties(true);
			Property prop = (Property) iter.next();
			assertEquals(prop.getValue(), null);
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
	
	/**
	 * Basic parsing tests
	 * @throws ResourceNotFoundException should not happen
	 * @throws ParserConfigurationException should not happen
	 * @throws SAXException should not happen
	 */
	public void testPort2() throws ResourceNotFoundException, SAXException, ParserConfigurationException {
		try {
			TemplateDescription ts0 = rep.getTemplateDescription("test-port2");
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
	
	/**
	 * Method testParsingEric0.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws KilimException :
	 */
	public void testParsingEric() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep.getTemplateDescription("eric/ClientImpl");
		//KilimTemplateViewer.viewTemplate(ts0);
	}

	/**
	 * Method testParsing2.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws KilimException :
	 */
	public void testParsing2() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep.getTemplateDescription("com/kelua/web framework/folders/folder with menu");
	}	

	/**
	 * Method testParsing3.
	 * @throws ResourceNotFoundException :
	 * @throws SAXException :
	 * @throws ParserConfigurationException :
	 * @throws KilimException :
	 */
	public void testParsing3() throws ResourceNotFoundException, SAXException, ParserConfigurationException, KilimException {
		TemplateDescription ts0 = rep.getTemplateDescription("main/all");		
		//KilimTemplateViewer.viewTemplate(ts0);	
		try {
			//this specific strategy should be defined in order to obtain the expected behaviour ...
			InstanciationStrategy strgy = new DefaultInstanciationStrategy();
			KilimConfiguration.setInstanciationStrategy(strgy);
			//Component compo = ComponentFactory.newComponent(ts0, strgy);	
			Component compo = ComponentFactory.newComponent(ts0);		
			//KilimComponentViewer.viewComponent(compo);			
		} catch (Exception ex) { 
			ex.printStackTrace();
			throw new InternalException(ex);
		}	
	}
}
