/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.description;

/**
 * @author horn
 * Describes a Port's arity. Three values are presently defined :
 * OPTIONAL : which means that a port can be unbound or bound to one element.
 * REQUIRED : which means that a port must be bound to one and only one element.
 * COLLECTION : which means that a port can be unbound or be bound to an arbitrary number of elements.
 * @see Port
 */

public final class Arity {
	/** the singleton representing an optional port */
	public static final Arity OPTIONAL = new Arity("?");
	/** the singleton representing a required port */
	public static final Arity REQUIRED = new Arity("1");
	/** the singleton representing a multiple port binding */
	public static final Arity COLLECTION = new Arity("*");
	
	private String description;
	
	/**
	 * The private constructor for arities.
	 * @param aDescription : a description which must be either "?" for optional, "1" for required, "*" 
	 * for collections.
	 */
	private Arity(String aDescription) {
		description = aDescription;
	}
	
	/**
	 * Returns a simple description of the arity. This description is either "?" (for OPTIONAL),
	 * "1" (for REQUIRED) or "*" (for COLLECTION) on the arity.
	 * @return String
	 */
	public String getDescription() {
		return description;
	}
}