/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.description;

/**
 * @author horn
 */
public interface BasicElement extends TemplateElement {
	
	/**
	 * Basic elements either provide values or exceute actions that complete the instatiation process This method. indicates whether
	 * the element returns a value in the Kilim model. The result is true for providers, ports, properties, inlined getters inlined news, constructors,
	 * methods which do not return void. It also return true for references to the elements previously defined (i.e. ports, properties, .).
	 * @return boolean
	 */
	boolean providesValue();
	
	/**
	 * Basic elements either provide values or exceute actions that complete the instatiation process This method. indicates whether	
	 * the element performs an action and can therefore been used in a trigger. The result is true for transformers, inlined setter, inlined news, methods, 
	 * and constructors. It also return true for references to the elements previously defined (setters, methods, ...).
	 * @return boolean
	 */
	boolean performsAction();
		
	/**
	 * indicates whether the element is an event source or not.
	 * @return boolean
	 */
	boolean isEventSource();
	
	
	/**
	 * returns the element kind. 
	 * @return int
	 */
	int getKind();
}
