package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public class ClassSource extends InlinedElement {
	private String className;
	
	/**
	 * The public constructor for a class source.
	 * @param aClassName : the name of the class.
	 * @param aTemplate : the template in which the class source is defined.
	 * @throws KilimException : generated when the className reference is null.
	 */
	public ClassSource(String aClassName, TemplateDescription aTemplate) throws KilimException {
		super(true, false, aTemplate);
		if (aClassName == null) {
			throw new KilimException("illegal null class name in template " + aTemplate.getName());
		}
		className = aClassName;
	}
	
	/**
	 * returns the name of the class source.
	 * @return String
	 */
	public String getClassName() {
		return className;
	}

	/**
	 * sets the name of the class source.
	 * @param aClassName : sets the class name.
	 * @throws KilimException : generated when the class name  is null.
	 */
	public void setClassName(String aClassName) throws KilimException {
		if (aClassName == null) {
			throw new KilimException("attempt to set a null class name to a class source " + className + " in template " + getContainingTemplate().getName());
		}
		className = aClassName;
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.CLASS;
	}
}
	
	/**
	 * Method setSupport has been defined as a rather high levelThis method should not be used. It always generates a KilimException.
	 * @param aElement : the element to be used as support (i.e. the object on which the method, constructor or .... will be invoked).
	 * @throws KilimException :
	 *
	public void setSupport(BasicElement aElement) throws KilimException {
		throw new KilimException("attempt to set a support for ClassSource " + className + " in template " + getContainingTemplate().getName());
	}
	
	/**
	 * Method getSupport.
	 * @return BasicElement
	 *
	public BasicElement getSupport() {
		return null;
	}
	
	/**
	 * 
	 * @return boolean
	 *
	public boolean isStatic() {
		return true;
	}

	*/