/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * this class provides null providers and transformers ...
 */
public class NullElement extends InlinedElement {
	/**
	* the public constructor of null elements. A null element is not considered as a property in Kilim since it can be used as a transformer.
	 * @param aTemplate : the template in which the element is declared.
	 * @param aName : the name of the element
	 * @param aStatus : the status of the element (should be one of KILIM.PUBLIC, KILIM.PROTECTED, KILIM.PRIVATE)
	 * @param isP  is true when the element provides a value (it behaves like a provider)
	 * @param isT is true when the element performs an action (it behaves like a transformer)
	 * @throws KilimException :
	 */
	public NullElement(String aName, int aStatus, boolean isP, boolean isT, TemplateDescription aTemplate) throws KilimException {
		super(isP, isT, aTemplate);
		setLocalName(aName);
	}

	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.NULL_ELEMENT;
	}
			
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#providesValue()
	 */
	public boolean providesValue() {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#performsAction()
	 */
	public boolean performsAction() {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}
	
	/**
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
			return this;
	}
}