package org.objectweb.kilim.description;

import java.util.Iterator;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * This is the parent interface for all interfaces and classes corresponding to template entites
 */
public abstract class TemplateElementImpl implements TemplateElement {
	private TemplateDescription containingTemplate;

	/**
	 * A public constructor for TemplateElementImpl.
	 * @param aTemplate : the template containing the element.
	 */
	protected TemplateElementImpl(TemplateDescription aTemplate) {
		containingTemplate = aTemplate;
	}
	
	/**
	 * @see java.lang.Object#Object()
	 */
	protected TemplateElementImpl() {
		this(null);
	}
		
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#getTemplate()
	 */
	public TemplateDescription getContainingTemplate() {
		return containingTemplate;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#getTemplateDefHierarchy()
	 */
	public Iterator getTemplateDefHierarchy() {
		try {
			return getContainingTemplate().getTemplateDefHierarchy(this);
		} catch (KilimException ex) {
			throw new InternalException(ex);
		}
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setContainingTemplate(TemplateDescription)
	 */
	public void setContainingTemplate(TemplateDescription aDescription) throws KilimException {
		containingTemplate = aDescription;
	}

	/**
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException ex) {
			System.err.println("Problem during clone operation of element " + this);
			return null;
		}
	}
}
