package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public class TpSetter extends TpAccessor {
	private BasicElement value;		

	/**
	 * The public constructor for a setter description.
	 * @param aSupport : the element on which the setter will b e applied.
	 * @param aField : the name of the field.
	 * @param jStatic : true if the setter is static.
	 * @param aTemplate : the template in which the setter is declared.
	 * @throws KilimException : generated if a Support, aField or aTemplate is null.
	 */
	public TpSetter(BasicElement aSupport, String aField, boolean jStatic, TemplateDescription aTemplate) throws KilimException {
		super(aSupport, aField, jStatic, false, true, aTemplate);
	}
	
	/**
	 * sets the value associated to the setter.
	 * @param aValue : the value to be set.
	 */
	public void setValue(BasicElement aValue) {
		value = aValue;
	}
	
	/**
	 * returns the value associated to the setter.
	 * @return BasicElement
	 */
	public BasicElement getValue() {
		return value;
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.SETTER;
	}
}