package org.objectweb.kilim.description;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */

public abstract class Trigger extends TemplateElementImpl {
	/** strings to be used to print event idents */
	public static final String[] EVENT_TYPE_NAME = { "Unbind" , "Bind"};
	
	/** integer constants used to identify bind events */
	public static final int BIND = 1;
	/** integer constants used to identify unbind events */
	public static final int UNBIND = 2;
	
	private String sourceName;
	private int kind;
	private List transformers;
		
	protected Trigger(String aName, int aKind) {
		sourceName = aName;
		kind = aKind;
	}
		
	/**
	 * returns the kind of the trigger event. It is one of TRIGGER.BIND or' TRIGGER.UNBIND.
	 * @return int
	 */
	public int getEventKind() {
		return kind;
	}
	
	/**
	 * @see org.objectweb.kilim.description.Trigger#getName()
	 */
	public String getSourceName() {
		return sourceName;
	}
		
	/**
	 * @see org.objectweb.kilim.description.Trigger#getName()
	 */
	public void setSourceName(String aName) {
		sourceName = aName;
	}

	/**
	 * @see org.objectweb.kilim.description.Trigger#addTransformer(Transformer)
	 */
	public void addTransformer(BasicElement aTransformer) throws KilimException {
		if (aTransformer == null) {
			throw new KilimException("illegal null value for transformer when added to a trigger in template " + getContainingTemplate().getName());
		}
		
		if (!aTransformer.performsAction()) {
			//throw new KilimException("attempt to add a non transformer to a trigger");
		}	
		
		String name = null;
		
		if (transformers == null) {
			transformers = new ArrayList();
		}

		if (transformers.contains(aTransformer)) {
				throw new KilimException("attempt to add an already defined transformer in a trigger in template " + getContainingTemplate().getName());
		}
		transformers.add(aTransformer);
	}
	
	/**
	 * @see org.objectweb.kilim.description.Trigger#removeTransformer(String)
	 */
	public void removeTransformer(BasicElement aTransformer) throws KilimException {
		if (aTransformer == null) {
			throw new KilimException("illegal null value for transformer when removed to a trigger in template " + getContainingTemplate().getName());
		}
		
		if (!aTransformer.performsAction()) {
			throw new KilimException("attempt to remove a non transformer from a trigger in template " + getContainingTemplate().getName());
		}
		
		if (transformers == null) {
			throw new KilimException("attempt to remove a transformer from an empty trigger in template " + getContainingTemplate().getName());
		}

		transformers.remove(aTransformer);
	}
		
	/**
	 * @see org.objectweb.kilim.description.Trigger#getTransformers()
	 */
	public Iterator getTransformers() {
		if (transformers == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return transformers.iterator();
	}
}