/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;

import org.objectweb.kilim.description.TemplateElementImpl;

/**
 * @author horn
 */
	
public interface ComponentElement {
	/**
	 * returns a reference to the component or the slot containing the element.
	 * It returns null when applied to the top level component.
	 * @return ContainerElement
	 */
	ContainerElement getContainingElement();
	
	/**
	 * returns a reference to the component containing the element.
	 * It returns null when applied to the top level component. It is equivalent to getContainingElement()
	 * for elements directly defined at a component level.
	 * It is equivalent to getContainingElement().getContainingElement() for elements defined at a slot level.
	 * @return Component
	 */
	Component getContainingComponent();
		
	/**
	 * returns the "template description" of the component element.
	 * @return TemplateElement
	 */
	TemplateElementImpl getElementDescription();
	
	/**
	 * returns all templates in which the element has been defined (and eventually redefined).
	 * This method is mainly targeted to debugging.
	 * @return Iterator
	 */
	Iterator getTemplateDefHierarchy();
	
	/**
	 * returns the fully qualified name of the component element.
	 * @return String : 
	 */
	String getQualifiedName();
	
	/**
	 * returns the local name of the component element.
	 * @return String
	 */
	String getLocalName();
}