/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;
import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
	
public interface ComponentSlot extends ContainerElement {
	/**
	 * retuns as an iterator the components plugged in the slot.
	 * @return Iterator
	 */
	Iterator getPlugIns();
	
	/**
	 * binds each port defined in the slot to the port having the same name in  the component (to be plugged).
	 * "Offered" ports in the slot are bound to "offered" interfaces in the component and the "required" ports in the component
	 * to the "required" ports in the slot.
	 * @param aComponent : the component to be plugged
	 * @throws KilimException : generated when aComponent is null or is already plugged in the slot.
	 */
	void plug(Component aComponent) throws KilimException;
	
	/**
	 * unbinds each port defined in the slot from the port having the same name in  the component (to be unplugged).
	 * @param aComponent : the component to be unplugged
	 * @throws KilimException :
	 */
	void unplug(Component aComponent) throws KilimException;
}