/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;
import java.util.List;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.TemplateElementImpl;

/**
 * @author horn.
 * This abstract class is the implementation of the interface ComponentElement, It is thus
 * the parent interface of all component element objects.It just contains a reference
 * to the containing element and to the template description. 
 */
	
public abstract class RtComponentElement implements ComponentElement, RuntimeElement {
	protected static long nbComponent = 0;
	protected static long nbElement = 0;
		
	public static long getNbComponent() {
		return nbComponent;
	}
	
	public static long getNbElement() {
		return nbElement;
	}
	private ContainerElement container;
	private TemplateElementImpl element;
	
	/**
	 * The protected constructor for a RtComponentElement.
	 * @param aElement : a reference to the template description of the element.
	 * @param aContainer : a reference to the containing element.
	 */
	protected RtComponentElement(TemplateElementImpl aElement, ContainerElement aContainer) {
		element = aElement;
		container = aContainer;
		nbElement++;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getContainingElement()
	 */
	public ContainerElement getContainingElement() {
		return container;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getContainingComponent()
	 */
	public Component getContainingComponent() {
		ContainerElement cont = getContainingElement();
		while (cont != null && !cont.isComponent()) {
			cont = cont.getContainingElement();
		}
		return (Component) cont;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getElementDescription()
	 */
	public TemplateElementImpl getElementDescription() {
		return element;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getTemplateDefHierarchy()
	 */
	public Iterator getTemplateDefHierarchy() {
		return element.getTemplateDefHierarchy();
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getQualifiedName()
	 */
	public String getQualifiedName() {
		if (container == null) {
			return getLocalName();
		}
		
		String contName = container.getQualifiedName();
		
		if ("".equals(contName)) {
			return getLocalName();
		}
		
		return contName + "/" + getLocalName();
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getTarget()
	 */
	public RuntimeElement getTarget() throws KilimException {
		return this;
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	protected boolean containsElement(List aList, String aName) throws InternalException {
		int lSize = aList.size();
		if (lSize == 0) {
			return false;
		}
		
		for (int i = 0; i < lSize; i++) {
			ComponentElement el = (ComponentElement) aList.get(i);
			if (aName.equals(el.getLocalName())) {
				return true;
			}
		}
		return false;
	}
	
	protected Object removeElement(List aList, String aName) throws InternalException {
		int lSize = aList.size();
		if (lSize == 0) {
			return null;
		}
		
		for (int i = 0; i < lSize; i++) {
			ComponentElement el = (ComponentElement) aList.get(i);
			if (aName.equals(el.getLocalName())) {
				return aList.remove(i);
			}
		}
		return null;
	}
	
	protected Object getElement(List aList, String aName) throws InternalException {
		int lSize = aList.size();
		if (lSize == 0) {
			return null;
		}
		
		for (int i = 0; i < lSize; i++) {
			ComponentElement el = (ComponentElement) aList.get(i);
			if (aName.equals(el.getLocalName())) {
				return el;
			}
		}
		return null;
	}			
}