/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;
import java.util.Stack;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.BasicElement;
import org.objectweb.kilim.description.InlinedElement;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.NamedElement;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.description.TemplateElementImpl;

/**
 * @author horn
 */
public abstract class RtComponentProvider extends RtComponentInterface {
	protected static Stack callStack = new Stack();
	private RuntimeSource support;
	private Object source;
		
	/**
	 * The public constructor for RtComponentProvider.
	 * @param aElement : the template description associated to the element
	 * @param aComponent : the component containing the provider (it is either a component or a slot).
	 * @param aSupport : the support of the provider, i.e. source supporting the method, the constructor, the getter, .... associated to the provider.
	 */
	protected RtComponentProvider(TemplateElementImpl aElement, ContainerElement aComponent, RuntimeSource aSupport) {
		super(aElement, aComponent);
		support = aSupport;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 */
	public String getLocalName() {
		TemplateElementImpl elem = getElementDescription();
		if (elem instanceof InlinedElement) {
			return ((InlinedElement) elem).getLocalName();
		} else if (elem instanceof NamedElement) {
			return ((NamedElement) elem).getLocalName();
		}
		return "";
	}

	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getEventSource()
	 */
	public Object getEventSourceValue() throws KilimException {
		return source;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#setEventSource(Object)
	 */
	public void setEventSourceValue(Object aSource) throws KilimException {
		source = aSource;
	}
	
	/**
	 * returns the support of the provider.
	 * @return RuntimeSource
	 */
	public RuntimeSource getSupport() {
		return support;
	}
		
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#bindProvider(RuntimeSource, boolean)
	 */
	public void bindProvider(RuntimeSource aSource, boolean jReplace) throws KilimException {
		TemplateDescription tmp0 = getContainingComponent().getTemplate();
		String lName = getLocalName();
		String tmpList = tmp0.getName();
		while ((tmp0 = tmp0.getSuperTemplate()) != null) {
			Iterator iter = tmp0.getBindings(lName, true);
			if (iter != KILIM.EMPTY_ITERATOR) {
				tmpList = tmpList + ", " + tmp0.getName();
			}
		}
		throw new KilimException("attempt to bind a provider " + getQualifiedName() + " (binding definition in templates  : " + tmpList + ")");
	}
	
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#unbindProvider(RuntimeSource)
	 */
	public void unbindProvider(RuntimeSource aSource) throws KilimException {
		throw new KilimException("attempt to unbind a provider " + getQualifiedName());
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isSingleValueInterface()
	 */
	public boolean isSingleValuePort() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isCollectionInterface()
	 */
	public boolean isCollectionPort() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isProvider()
	 */
	public boolean isProvider() {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isProperty()
	 */
	public boolean isProperty() {
		return false;
	}
			
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#specificBindValue(Object)
	 */
	protected void specificBindValue(Object aValue)  throws KilimException {
		throw  new KilimException ("attempt to bind a value to a  provider " + getQualifiedName());
	}
	
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#specificUnbindValue()
	 */
	protected void specificUnbindValue()  throws KilimException {
		throw  new KilimException ("attempt to bind a value to a  provider " + getQualifiedName());
	}
		
	/**
	 * Method getKind. This method returns the kind of the provider.
	 * @return int
	 */
	public int getKind() {
		return ((BasicElement) getElementDescription()).getKind();
	}
}