/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Stack;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
	
public interface RuntimeAction extends RuntimeElement {		
	/**
	 * executes the method, constructor, setter, getter implementing the RuntimeAction.
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references 
	 * or on illegal elements.
	 */
	void execute() throws KilimException;
	
	/**
	 * returns whether an action can be performed without any new constructive action.
	 * @param exClude is the stack of runtime element  which are under evaluation (i.e. element in the process of a constructive action).
	 * @return boolean
	 * @throws KilimException :
	 */
	boolean checkAction(Stack exClude) throws KilimException;
	
	/**
	 * returns the target object. This method has been introduced to deal with references. 
	 * @return RuntimeSource
	 * @throws KilimException : generated when invoked on an uninitialized references.
	 */
	RuntimeElement getTarget() throws KilimException;
	
	/**
	 * sets the event source value (i.e. the reference to the object being the current event source)..
	 * @param aValue : the value of the source.
	 * @throws KilimException : the exception is generated when the provided value is badly typed or
	 * when the component element is not bound
	 */
	void setEventSourceValue(Object aValue) throws KilimException;
	
	/**
	 * gets the event source value (i.e. the reference to the object being the current event source)..
	 * @return Object : the reference of the trigger source.
	 * @throws KilimException : the exception is generated when the method is invoked on unbound references 
	 * or on illegal elements.
	 */
	Object getEventSourceValue() throws KilimException;
}
