/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.Port;
import org.objectweb.kilim.description.Slot;
import org.objectweb.kilim.model.services.DefaultNamingContext;

/**
 * @author horn
 */
public class SlotFactory extends DefaultNamingContext implements Factory {
	private String localName;	
	private ComponentFactory containingFactory;	
	
	private Slot slotDescription;
	private Component containingComponent;
	
	/**
	 * The public constructor for slot factories.
	 * @param aName :
	 * @param aFactory :
	 * @throws KilimException :
	 */

	public SlotFactory(String aName, ComponentFactory aFactory) throws KilimException {
		super(aName, aFactory);
		localName = aName;
		containingFactory = aFactory;
		containingComponent = aFactory.getComponent();
		if (containingFactory != null) {
			containingFactory.addChildNamingContext(aName, this);
		}
	}
	
	/**
	 * returns the containing factory.
	 * @return ComponentFactory
	 */
	public ComponentFactory getContainingFactory() {
		return containingFactory;
	}
		
	/**
	 * adds a SubFactory (should not be used !!)
	 * @param aElement :
	 * @throws KilimException : always generated since slots are terminal contrainers in the present model.
	 */
	public void addSubFactory(ComponentFactory aElement) throws KilimException {
		throw  new KilimException("attempt to add a subfactory to slot factory " + getQualifiedName());
	}
	
	/**
	 * removes a SubFactory  (should not be used !!).
	 * @param aElement :
	 * @throws KilimException : always generated since slots are terminal contrainers in the present model.
	 */
	public void removeSubFactory(ComponentFactory aElement) throws KilimException {
		throw  new KilimException("attempt to remove a subfactory from slot factory " + getQualifiedName());		
	}
	
	/**
	 * returns a subFactory  (should not be used !!).
	 * @param aName :
	 * @return ComponentFactoryElement
	 * @throws KilimException : always generated since slots are terminal contrainers in the present model.
	 */
	public ComponentFactory getSubFactory(String aName) throws KilimException {
		throw  new KilimException("attempt to get subfactory " + aName + " from slot factory " + getQualifiedName());		
	}
	
	/**
	 * Method getComponentFactorys.
	 * @return Iterator
	 * @throws KilimException :
	 */
	public Iterator getSubFactories() throws KilimException {
		throw  new KilimException("attempt to get subfactories from slot factory " + getQualifiedName());		
	}
		
	/**
	 * creates a new runtime descriptor of a slot from an abstract descriptor of a slot.
	 * @param aSlot : the abstract descriptor of a slot
	 * @return RuntimeSlot
	 * @throws KilimException : generated if aSlot is null.
	 */
	public RtComponentSlot newSlot(Slot aSlot) throws KilimException {
		if (aSlot == null) {
			throw new KilimException("attempt to create a null slot in component " + containingComponent.getQualifiedName());
		}
		
		RtComponentSlot rSlot = new RtComponentSlot(aSlot, containingComponent, this);
		//((RtComponent) containingComponent).addSlot(rSlot);
		Iterator iter = aSlot.getPorts();
		while (iter.hasNext()) {
			Port elem = (Port) iter.next();
			RtComponentInterface interf = newInterface(elem, rSlot);
			rSlot.addInterface(interf);
			addBoundName(interf.getLocalName(), interf);
		}
		return rSlot;
	}
	
	private RtComponentInterface newInterface(Port port, RtComponentSlot aSlot) throws KilimException {
		RtComponentInterface interf = null;
		if (port.isUnary()) {
			interf = new RtSingleValuePort(port, aSlot);
		} else {
			interf = new RtCollectionPort(port, aSlot);
		}

		String interfName;
		String qualName = getQualifiedName();
		if ("".equals(qualName)) {
			interfName = port.getLocalName();
		} else {
			interfName = qualName + "/" + port.getLocalName();
		}
		return interf;
	}
}