/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.services;

import java.util.HashMap;
import java.util.Iterator;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;

/**
 * Thids class is a simple container for annotations (i.e. a couple made of a name and a value).
 * @author horn
 */

public class Annotations {

	private HashMap externalServices;
	
	/**
	 * @see java.lang.Object#Object()
	 */
	public Annotations() { }
		
	/**
	 * adds a new annotation
	 * @param aName : the name of the annotation to be added. 
	 * @param xService : the reference of the object offering the annotation
	 * @throws KilimException : the exception  is generated if the name or the reference is null or if the annotation 
	 * has already been added (It should be removed first).
	 */
	public void addAnnotation(String aName, Object xService) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to add an annotation with a null name");
		}
		if (xService == null) {
			throw new KilimException("attempt to add a null annotation");
		}			
		if (externalServices == null) {
			externalServices = new HashMap();
		}		
		externalServices.put(aName, xService);
	}
	
	/**
	 * removes an annotation
	 * @param aName : the name of the annotation to be removed.
	 * @throws KilimException : the exception is generated if the name is null or if the annotation name is unknown.
	 */
	public void removeAnnotation(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to remove an annotation through a null name");
		}		
		if (externalServices == null) {
			throw new KilimException("attempt to remove an unknown annotation " + aName);
		}	
		Object result = externalServices.remove(aName);
		if (result == null) {
			throw new KilimException("attempt to remove an unknown annotation " + aName);
		}		
	}
	
	/**
	 * returns an annotation.
	 * @param aName : the name of the annotation to be returned.
	 * @return Object : the reference of the object offering the annotation.
	 * @throws KilimException : the exception is generated if the name is null.
	 */
	public Object getAnnotation(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get an external service with a null name from the instanciation strategy");
		}		
		if (externalServices == null) {
			return null;
		}
		return externalServices.get(aName);
	}
	
	/**
	 * returns an iterator containing the names of all the currently added annotations.
	 * @return Iterator
	 */
	public Iterator getAnnotationNames() {
		if (externalServices == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return externalServices.keySet().iterator();
	}	
}
