/**
 * Kilim
 * Copyright (C) 2001 Kelua SA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * Contact: kilim@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * -----------------------------------------------------------------
 * $Id: ClassPathFinderImpl.java,v 1.1.1.1 2002/08/06 13:13:49 bmi Exp $
 * -----------------------------------------------------------------
 */

package org.objectweb.kilim.model.services;

import java.lang.reflect.Array;
import java.util.HashMap;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.Type;

/**
 * @author horn
 */
public class DefaultRuntimeClassLoader implements RuntimeClassLoader {
   private HashMap caches;

	/** instance is a singleton associated to a default runtime manager */
   public static RuntimeClassLoader instance = new DefaultRuntimeClassLoader();
     
	/**
	 * @see java.lang.Object#Object()
	 */
   public DefaultRuntimeClassLoader() {
      caches = new HashMap();
   } 
      
	/**
	 * @see org.objectweb.kilim.model.RuntimeManager#getClass(Type)
	 */
   public Class getClass(Type type) throws KilimException {
      return getClass(type.getName());
   }
  
	/**
	 * @see org.objectweb.kilim.model.RuntimeManager#getClass(String)
	 */
   public Class getClass(String class_name) throws KilimException {
      Class result = null;
      ClassLoader class_loader = Thread.currentThread().getContextClassLoader();
      Object cl_cache = caches.get(class_loader);
      HashMap class_loader_cache = null;
      if (cl_cache == null) {
         class_loader_cache = new HashMap();
         caches.put(class_loader, class_loader_cache);
      } else {
         class_loader_cache = (HashMap) cl_cache;
         result = (Class) class_loader_cache.get(class_name);
      }

      if (result != null) {
         return result;
      }

      int size = 0;
      int index = 0;
      while (true) {
         index = class_name.indexOf('[', index);
         if (index < 0) {
            break;
         } else {
            size++;
            //index = class_name.indexOf('[',index + 2);
            index += 2;
         }
      }
      if (size > 0) {
         // class_name is an array type. Try to find the element type first.
         String element_class_name = class_name.substring(0, class_name.indexOf('['));
         Class element_class = getClass(element_class_name);
         // create an array instance
         int[] sizes = new int[size];
         for (int i = 0; i < size; i++) {
            sizes[i] = 0;
         }
         Object array = Array.newInstance(element_class, sizes);
         // return its class
         result = array.getClass();
      } else {
         result = primitiveClass(class_name);
        
         if (result == null) {
            try {
               result = class_loader.loadClass(class_name);
            } catch (ClassNotFoundException ignored) { }
            if (result == null) {
               try {
                  result = Class.forName(class_name);
               } catch (ClassNotFoundException e) {
                  throw new KilimException(e);
               }
            }
         }
      }
      class_loader_cache.put(class_name, result);
     
      return result;
   }

   static Class primitiveClass(String name) {
      switch (name.charAt(0)) {
         case 'b':
            switch (name.charAt(1)) {
               case 'o':
                  return "boolean".equals(name) ? boolean.class : null;
               default:
                  return "byte".equals(name) ? byte.class : null;
            }
         case 'c':
            return "char".equals(name) ? char.class : null;
         case 'd':
            return "double".equals(name) ? double.class : null;
         case 'f':
            return "float".equals(name) ? float.class : null;
         case 'i':
            return "int".equals(name) ? int.class : null;
         case 'l':
            return "long".equals(name) ? long.class : null;
         case 's':
            return "short".equals(name) ? short.class : null;
         default:
            return "void".equals(name) ? void.class : null;
      }
   }
}
