/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.services;

import java.util.LinkedHashMap;
import java.util.Iterator;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;

/**
 * This class was initially dedicated to the definition of instanciation strategies. It has been generalized to
 * be a generic container for any external references which may be accessed through a kilim component.
 * @author horn
 */

public class ExternalValueReferences {
	private LinkedHashMap externalReferences;
	
	/**
	 * @see java.lang.Object#Object()
	 */
	public ExternalValueReferences() { }
	
	/**
	 *This method adds a new external reference.
	 * @param aName : the name of the external reference.
	 * @param aObject : the referenced object.
	 * @throws KilimException : generated if aName is null.
	 */
	public void addExternalValueReference(String aName, Object aObject) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to add an external reference with a null name in the external value mapper");
		}
		
		if (aObject == null) {
			throw new KilimException("attempt to add an null external reference in the external value mapper");
		}
		if (externalReferences == null) {
			externalReferences = new LinkedHashMap();
		}
		externalReferences.put(aName, aObject);
	}
	
	/**
	 * removes an external reference.
	 * @param aName : the name of the reference to be removed.
	 * @throws KilimException : generated  if aName is null.
	 */
	public void removeExternalValueReference(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to add an external reference with a null name in the external value mapper");
		}
		if (externalReferences == null) {
			return;
		}
		externalReferences.remove(aName);
	}

	/**
	 * returns an external reference.
	 * @param aName : the name of the external reference
	 * @return Object : the referenced object 
	 * @throws KilimException : generated if aName is null.
	 */
	public Object getExternalValueReference(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to add an external reference with a null name in the iexternal value mapper");
		}
		if (externalReferences == null) {
			return null;
		}
		return externalReferences.get(aName);
	}
	
		/**
		 * returns as an iterator the names of known external references.
		 * @return Iterator
		 */
		public Iterator getExternalReferenceNames() {
		if (externalReferences == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return externalReferences.keySet().iterator();
	}	
}
