/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.repository;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.TemplateDescription;


/**
 * Kilim template repository. The interface implemented by all tools reading template descriptions.
 * @author horn
 */
public interface Repository {
	
	/** The file extension of XML Kilim configuration files */
	public static final String KILIM_FILES_EXTENSION = ".kilim";
	
	/**
	 * Sets the repository resourceLoader.
	 * @param aResourceLoader : The resourceLoader to set
	 */
	void setResourceLoader(ResourceLoader aResourceLoader);
	
	/**
	 * Returns the current repository resourceLoader.
	 * @return ResourceLoader
	 */
	ResourceLoader getResourceLoader();

   /**
    * Method getTemplateDescription.
    * @param resourceName : the name of the template to be loaded
    * @return Template the loaded description template
    * @throws ResourceNotFoundException if a required template can't be found
    * @throws KilimException :
    */
   TemplateDescription getTemplateDescription(String resourceName) throws ResourceNotFoundException, KilimException;
}
