package org.objectweb.kilim.tools;

import java.util.LinkedList;
import java.util.Stack;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import org.objectweb.kilim.description.KILIM;

/**
 * @author horn
 */
public class KilimTraceTreeModel implements TreeModel {			//extends DefaultTreeModel {
	private String rootStr = "trace start";
	private KilimTraceTreeNode root = new KilimTraceTreeNode(rootStr);
	private Stack nodeStack = new Stack();
	private KilimTraceTreeNode current;

	/**
	 * Method KilimComponentTreeModel.
	 */
	public KilimTraceTreeModel() {
		super();
		root = new KilimTraceTreeNode(rootStr);
		nodeStack.push(root);
		current = root;
	}

	/**
	 * @see javax.swing.tree.TreeModel#getRoot()
	 */
	public Object getRoot() {
		return root;
	}

	/**
	 * @see javax.swing.tree.TreeModel#getChild(Object, int)
	 */
	public Object getChild(Object aParent, int index) {
		if (aParent == null) {
			return null;
		}

		LinkedList list = ((KilimTraceTreeNode) aParent).getChildren();
		if (list == null || list == KILIM.EMPTY_LIST) {
			return null;
		}
		if (index < 0 || index >= list.size()) {
			return null;
		}
		return list.get(index);
	}

	/**
	 * @see javax.swing.tree.TreeModel#getChildCount(Object)
	 */
	public int getChildCount(Object aParent) {
		if (aParent == null) {
			return 0;
		}

		LinkedList list = ((KilimTraceTreeNode) aParent).getChildren();
		if (list == null || list == KILIM.EMPTY_LIST) {
			return 0;
		}
		return list.size();
	}

	/**
	 * @see javax.swing.tree.TreeModel#isLeaf(Object)
	 */
	public boolean isLeaf(Object node) {
		return (getChildCount(node) == 0);
	}

	/**
	 * @see javax.swing.tree.TreeModel#valueForPathChanged(TreePath, Object)
	 */
	public void valueForPathChanged(TreePath path, Object newValue) {
	}

	/**
	 * @see javax.swing.tree.TreeModel#getIndexOfChild(Object, Object)
	 */
	public int getIndexOfChild(Object aParent, Object child) {
		if (aParent == null) {
			return -1;
		}

		LinkedList list = ((KilimTraceTreeNode) aParent).getChildren();
		if (list == null || list == KILIM.EMPTY_LIST) {
			return -1;
		}		
		int lSize = list.size();
		for (int i = 0; i < lSize; i++) {
			if (list.get(i) == child) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * @see javax.swing.tree.TreeModel#addTreeModelListener(TreeModelListener)
	 */
	public void addTreeModelListener(TreeModelListener l) {
	}

	/**
	 * @see javax.swing.tree.TreeModel#removeTreeModelListener(TreeModelListener)
	 */
	public void removeTreeModelListener(TreeModelListener l) {
	}
	
	/**
	 * Method addChild.
	 * @param aParent :
	 * @param aChild :
	 */
	public void addChild(Object aParent, Object aChild) {		
		KilimTraceTreeNode node1 = new KilimTraceTreeNode(aChild);
		current.addChild(node1);
		nodeStack.push(node1);
		current = node1;
	}
	
	/**
	 * Method setSeqNumber.
	 * @param aObject :
	 * @param aNumber :
	 * @param aMode :
	 */
	public void setSeqNumberAndMode(Object aObject, int aNumber, int aMode) {
		KilimTraceTreeNode node = (KilimTraceTreeNode) nodeStack.pop();
		node.setSeqNumberAndMode(aNumber, aMode);
		current = (KilimTraceTreeNode) nodeStack.lastElement();
	}		
}

class KilimTraceTreeNode {
	private int seqNumber;
	private int mode;
	private Object value;
	private LinkedList children;
	
	public KilimTraceTreeNode(Object aValue) {
		value = aValue;
	}
	
	public void setSeqNumberAndMode(int aNumber, int aMode) {
		seqNumber = aNumber;
		mode = aMode;
	}
	
	public int getSeqNumber() {
		return seqNumber;
	}
	
	public int getMode() {
		return mode;
	}
	
	public void addChild(KilimTraceTreeNode aNode) {
		if (children == null) {
			children = new LinkedList();
		}
		children.addLast(aNode);
	}
	
	public int getChildCount() {
		if (children == null) {
			return 0;
		}
		return children.size();
	}
	
	public LinkedList getChildren() {
		if (children == null) {
			return KILIM.EMPTY_LIST;
		}
		return children;
	}
	
	public Object getValue() {
		return value;
	}
}