package org.objectweb.kilim.tools;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.BasicElement;
import org.objectweb.kilim.description.ClassSource;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.TpConstructor;
import org.objectweb.kilim.description.TpGetter;
import org.objectweb.kilim.description.TpMethod;
import org.objectweb.kilim.model.RtCollectionPort;
import org.objectweb.kilim.model.RtComponentProperty;
import org.objectweb.kilim.model.RtComponentProvider;
import org.objectweb.kilim.model.RtSingleValuePort;
import org.objectweb.kilim.model.mapping.TreeModelMapper;

/**
 * @author delpiano, horn
 */
public class KilimTraceTreeViewer {
	
	private static final String WINDOW_NAME = "Kilim Trace Viewer";
	
	private static ActionListener action_listener = new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
			synchronized (this) {
				this.notifyAll();
			}
		}
	};
	
	private static JFrame tree_frame = new JFrame();
	private static JFrame text_frame = new JFrame();
	private static JTextArea textArea;
	
	static {
		tree_frame.setTitle(WINDOW_NAME);
		tree_frame.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent evt) {
				synchronized (action_listener) {
					action_listener.notifyAll();
				}
			}
		});

		text_frame.setTitle(WINDOW_NAME);
		text_frame.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent evt) {
				synchronized (action_listener) {
					action_listener.notifyAll();
				}
			}
		});
	}
	
	private static JTree tree = new JTree();
	private static KilimTraceTreeModel treeModel;
		
	/**
	 * Method viewComponent.
	 * @param aModel :
	 * @param aTextArea :
	 */
	public static void viewTrace(KilimTraceTreeModel aModel, JTextArea aTextArea) {
		textArea = aTextArea;
		tree.setCellRenderer(new KilimTraceTreeCellRenderer());	
		Container cntnr = tree_frame.getContentPane();
		cntnr.setLayout(new BorderLayout());
		JScrollPane scrllpn = new JScrollPane(tree);
		cntnr.add(BorderLayout.CENTER, scrllpn);
		tree_frame.setSize(650, 800);
		tree_frame.setLocation(50, 50);
		ToolTipManager.sharedInstance().registerComponent(tree);
		JButton button = new JButton("continue");
		button.addActionListener(action_listener);
		cntnr.add(BorderLayout.SOUTH, button);

		treeModel = aModel;
		tree.setModel(treeModel);
		tree.repaint();

		Container textCont = text_frame.getContentPane();
		textCont.setLayout(new BorderLayout());
		JScrollPane textSPane = new JScrollPane(aTextArea);
		textCont.add(BorderLayout.CENTER, textSPane);
		text_frame.setSize(600, 800);
		text_frame.setLocation(700, 50);
		textArea.append("\n// -----------------------END VIEW-----------------------");
		textArea.repaint();
				
		tree_frame.setVisible(true);
		text_frame.setVisible(true);

		synchronized (action_listener) {
			try {
				action_listener.wait();
			} catch (InterruptedException exp) {
				exp.printStackTrace();
			}
		}
	}
	
	/**
	 * Method updateView.
	 */
	public static void updateView() {
		tree.setModel(null);
		tree.setModel(treeModel);		
		tree_frame.validate();
		tree_frame.repaint();
		textArea.append("\n// -----------------------END VIEW-----------------------");
		textArea.repaint();

		synchronized (action_listener) {
			try {
				action_listener.wait();
			} catch (InterruptedException exp) {
				exp.printStackTrace();
			}
		}
	}
}

class KilimTraceTreeCellRenderer extends DefaultTreeCellRenderer {
	
	private  static final String TRACE_START_ICON_NAME =  "start.gif";
	private static Icon TRACE_START_ICON = null;

	private static final String[] ICON_NAMES = { "provider.gif" , "transformer.gif" , "property.gif" , "trigger.gif", "external.gif" };											
	private static  Icon[] ICONS = new Icon[ICON_NAMES.length];
	
	static {
		try {
			TRACE_START_ICON = getIcon(TRACE_START_ICON_NAME);
			for (int i = 0; i < ICON_NAMES.length; i++) {
				ICONS[i] = getIcon(ICON_NAMES[i]);
			}
		} catch (Exception ex) {
			System.err.println("problem when getting the start icon ");
		}
	}
	
	/**
	 * @see javax.swing.tree.TreeCellRenderer#getTreeCellRendererComponent(JTree, Object, boolean, boolean, boolean, int, boolean)
	 */
	public java.awt.Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {															
		super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);

		if (value instanceof String) {
			if (TRACE_START_ICON != null) {
				setIcon(TRACE_START_ICON);
			}
			setText("Start");
			return this;
		}
		
		if (!(value instanceof KilimTraceTreeNode)) {
			return this;
		}
		
		KilimTraceTreeNode node = (KilimTraceTreeNode) value;
		Object object = node.getValue();
		if (object instanceof String) {
			setText("+++++++++++++START TRACE+++++++++++++");
			return this;
		}
		
		int seqNumber = node.getSeqNumber();
		int mode = node.getMode();
		String addtnl_text = "undefined";	
		
		if (object instanceof RtComponentProperty) {
			RtComponentProperty prop = (RtComponentProperty) object;
			if (ICONS[2] != null) {
					setIcon(ICONS[2]);
			}
			addtnl_text = "property : " + prop.getQualifiedName();
			setToolTipText(addtnl_text);
			String pValue ;
			try {
				 pValue = "" + prop.getValue();
			} catch (KilimException ex) {
				pValue = "undefined property";
			}				
			setText("<" + seqNumber + " > " + prop.getQualifiedName() + " [" + pValue + "]");
			return this;
		}
		
		if (object instanceof RtComponentProvider) {
			RtComponentProvider elem = (RtComponentProvider) node.getValue();
			int kind = elem.getKind();
			
			BasicElement desc = (BasicElement) elem.getElementDescription();
			if (mode == TreeModelMapper.SOURCE) {
				if (ICONS[0] != null) {
					setIcon(ICONS[0]);
					if (object instanceof RtSingleValuePort) {
						addtnl_text = "1-ary port belonging to " + ((RtSingleValuePort) object).getContainingElement().getQualifiedName();
					} else if (object instanceof RtCollectionPort) {
						addtnl_text = "n-ary port belonging to " + ((RtCollectionPort) object).getContainingElement().getQualifiedName();
					} else {
						addtnl_text = "provider belonging to " + ((RtComponentProvider) object).getContainingElement().getQualifiedName();
					}
					setToolTipText(addtnl_text);
				}
			} else {
				if (ICONS[1] != null) {
					setIcon(ICONS[1]);
					addtnl_text = "transformer belonging to " + ((RtComponentProvider) object).getContainingElement().getQualifiedName();
					setToolTipText(addtnl_text);
				}	
			}			
						
			switch(kind) {
				case KILIM.GETTER :
					TpGetter gDesc = (TpGetter) desc;
					String fName = gDesc.getFieldName();
					setText("<" + seqNumber + " > " + "getter of field " + fName);
					break;
				case KILIM.CONSTRUCTOR :
					TpConstructor nDesc = (TpConstructor) desc;
					setText("<" + seqNumber + " > " + "new ");
					break;
				case KILIM.METHOD :
					TpMethod mDesc = (TpMethod) desc;
					String mName = mDesc.getMethodName();
					setText("<" + seqNumber + " > " + "method " + mName);
					break;
				case KILIM.CLASS :
					ClassSource cDesc = (ClassSource) desc;
					String cName = cDesc.getClassName();
					setText("<" + seqNumber + " > " + "class " + cName);
					break;
				case KILIM.NULL_ELEMENT :
					setText("<" + seqNumber + " > " + "null element");
					break;
				case KILIM.REFERENCE :
					setText("<" + seqNumber + " > " + "reference ");
					break;
				default :
					break; 
			}
			
			return this;
		}
		
		//TpSetter sDesc = (TpSetter) desc;
		//String fName = sDesc.getFieldName();
		//setText("getter of field " + fName);
		setIcon(ICONS[4]);
		setText("<" + seqNumber + " > " + "undefined element of class " + object.getClass().getName());			
		return this;
	}

	private static ImageIcon getIcon(String name) {
		URL url = ClassLoader.getSystemClassLoader().getSystemResource(name);
		if (url != null) {
			Image img = Toolkit.getDefaultToolkit().getImage(url);
			if (img != null) {
				return new ImageIcon(img);
			} else {
				return null;
			}
		} else {
			return null;
		}
	}
}
